#include <string>
#include <vector>
#include "RefCountingPointer.h"
#include "ServiceProxy.h"
#include "ServiceReference.h"
#include "sdo.h"

/*
 * Copyright(C) OASIS(R) 2006,2009. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 */

/** @file */  
 
#ifndef _oasis_sca_ComponentContext_h_
#define _oasis_sca_ComponentContext_h_

namespace oasis {
    namespace sca {
            
        class ComponentContext;      
        /**
         * RefCounting Pointer for a ComponentContext.
         */
        typedef RefCountingPointer<ComponentContext> ComponentContextPtr;
 
        /**
         * Defines the ComponentContext interface
         */
        class ComponentContext {
            public:
   
                /**
                 * Return a ComponentContext object for the current component.
                 * @return ComponentContext for the current component.
                 */ 
                static ComponentContextPtr getCurrent();
                
        
                /**
                 * Returns the absoulte URI for this component
                 * @return Absolute URI for the current component.
                 */ 
                virtual std::string getURI();
                
                
        
                /**
                 * Used to get a service proxy implementing the interface defined for a Reference.
                 * @param referenceName Name of the Reference to get an interface object for
                 * @return Pointer to a ServiceProxy implementing the interface of the Reference.  
                 *         This will be NULL if referenceName is not defined for the component.
                 * @throws MultipleServicesException if the reference resolves to more than one service.
                 */ 
                virtual ServiceProxyPtr getService(const std::string& referenceName);
                
        
                /**
                 * Used to get a vector of service proxies implementing the interface defined for a Reference.
                 * @param referenceName Name of the Reference to get an interface object for
                 * @return Vector of pointers to ServiceProxy objects implementing the interface of the Reference.
                 *         This vector will be empty if referenceName is not defined for the component.
                 */ 
                virtual std::vector<ServiceProxyPtr> getServices(const std::string& referenceName);
                
                
        
                /**
                 * Used to get a ServiceReference for a Reference.
                 * @param referenceName Name of the Reference to get an interface object for
                 * @return ServiceReference for the Reference.  
                 *         This will be NULL if referenceName is not defined for the component.
                 * @throws MultipleServicesException if the reference resolves to more than one service
                 */ 
                virtual ServiceReferencePtr getServiceReference(const std::string& referenceName);
                
        
                /**
                 * Used to to get a vector of ServiceReferences for a Reference.
                 * @param referenceName Name of the Reference to get an interface object for
                 * @return Vector of ServiceReferences for the Reference.
                 *         This vector will be empty if referenceName is not defined for the component.
                 */ 
                virtual std::vector<ServiceReferencePtr> getServiceReferences(const std::string& referenceName);
                                
                
        
                /**
                 * Used to get a component's configured property values.
                 * @return An SDO from which all the properties defined in the componentType file can be retrieved.
                 */ 
                virtual DataObjectPtr getProperties();
                
        
                /**
                 * Used to get its an SDO DataFactory which can be used to create DataObjects for complex data types
                 * used by this component.
                 * @return An SDO DataFactory which has definitions for all complex data types used by a component
                 */ 
                virtual DataFactoryPtr getDataFactory();
                
                
        
                /**
                 * Used to get a ServiceReference for use with some callback APIs.
                 * @return A ServiceReference for the service provided by this component
                 * @throws MultipleServicesException if the component implements more than one Service
                 */ 
                virtual ServiceReferencePtr getSelfReference();
                
        
                /**
                 * Dereferences the underlying pointer
                 * @param serviceName Name of the Service to get a ServiceReference for.
                 * @return A ServiceReference for the named service provided by this component.
                 */ 
                virtual ServiceReferencePtr getSelfReference(const std::string& serviceName);
            };    
    
    } // namespace sca
} // namespace oasis 

#endif /* _oasis_sca_ComponentContext_h_ */
