/*
 * Copyright(C) OASIS(R) 2006,2009. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.
 */
 
/** @file */  
 
#ifndef _oasis_sca_RefCountingPointer_h_
#define _oasis_sca_RefCountingPointer_h_

namespace oasis {
    namespace sca {
            
        template <typename T>
        
        /**
         * A smart pointer class that holds a real (dumb) pointer to the object.
         * If the reference counting pointer is copied, then a duplicate pointer is returned with the same real pointer.  
         * A reference count within the object is incremented for each copy of the pointer, so only when all pointers
         * go out of scope will the object be freed.
         */
        class RefCountingPointer {
            public:
        
                /**
                 * Dereferences the underlying pointer
                 * @return A reference to the value of the pointer.
                 * @throws SCANullPointerException if the pointer is NULL
                 */ 
                T& operator* () const;
        
                /**
                 * Used to invoke member functions on the underlying pointer. This is equivalent to invoking p->func() 
                 * where func() is a member function defined on the underlying pointer type. 
                 * @throws SCANullPointerException if the pointer is NULL
                 */
                T* operator-> () const;
                
                /**
                 * Used to determine if the underlying pointer is set, i.e. if (p) 
                 * @return Zero if the underlying pointer is null, otherwise a non-zero value.
                 */
                operator void* () const;
                
                /**
                 * Used to determine if the underlying pointer is not set, i.e. if (!p) 
                 * @return A non-zero value if the underlying pointer is null, otherwise zero.
                 */
                bool operator! () const;
        };
 
        
        template <typename T, typename U>
        /**
         * Provides the semantic behavior of the const_cast operator for use with RefCountingPointers
         * @param other Pointer to cast
         * @return A RefCountingPointer instance templatized on the target type.
         */
        RefCountingPointer<T> constCast(RefCountingPointer<U> other);
        

        template <typename T, typename U>
        /**
         * Provides the semantic behavior of the dynamic_cast operator for use with RefCountingPointers
         * @param other Pointer to cast
         * @return A RefCountingPointer instance templatized on the target type.
         *         This may return an invalid RefCountingPointer instance if the cast fails. 
         */
        RefCountingPointer<T> dynamicCast(RefCountingPointer<U> other);
        
 
        template <typename T, typename U>
        /**
         * Provides the semantic behavior of the reinterpret_cast operator for use with RefCountingPointers
         * @param other Pointer to cast
         * @return A RefCountingPointer instance templatized on the target type.
         */
        RefCountingPointer<T> reinterpretCast(RefCountingPointer<U> other);
        

        template <typename T, typename U>
        /**
         * Provides the semantic behavior of the static_cast operator for use with RefCountingPointers
         * @param other Pointer to cast
         * @return A RefCountingPointer instance templatized on the target type.
         */
        RefCountingPointer<T> staticCast(RefCountingPointer<U> other);

    
    } // namespace sca
} // namespace oasis 

#endif /* _oasis_sca_RefCountingPointer_h_ */
